;*************************************************************************************
;
;
;				Title: RF remote control at 418MHz with roll-code
;               
;					(RF 418MHz receiver code)
;
;				Device:		AT90s2313 with 4MHz crystal
;							and RLP-434 RF receiver (418 MHz)
;				Author:		Serasidis Vasilis
;				Date:		04.06.2004
;				Updated:		15.04.2005 Added roll-code feaure.
;				Home:		http://www.serasidis.gr
;				email:		info@serasidis.gr
;
;				Based in hardware UART at 2400bps, 8n1
;
;***************************************************************************************



.include "2313def.inc"			;Define chip 

.equ	BAUD		=103 		;2400 bps at 4.00 MHz.
.equ	first_byte	=$30

.def	temp		=R16		;temporary storage register.
.def	temp2		=R17
.def	rxbyte		=R19		;Received data.

.def	command		=R20
.def	init_byte	=R21
.def	second_byte	=R22
.def	enc_second_byte=R25
.def	output		=R23
.def	bitcnt		=R24		;bit counter
.def	backup		=R26
.def	dcount		=R27
.def	dcount2		=R28


;=======================================================================


.cseg

.org	0
	
	rjmp	reset			;When power-on the circut, go to reset.

.org	$007 				
	rjmp	start			; awake from sleep when UART RX is complete...
							;... and go to start.


reset:
	ldi		temp,RAMEND
	out		SPL,temp		;Init Stack Pointer

	ldi		temp,0b11111111 ;port B output
	out		DDRB,temp

	ldi		temp,0b11110000	;PD 1 input, others output
	out		DDRD,temp

	ldi		temp,BAUD
	out		UBRR,temp		;Set baud rate generator


;===========   Set UART, as Transmiter and receiver   =========

	ldi		temp,0b00011000
	out		UCR,temp		;Enable UART Tx and Rx
;==============================================================

	clr		output
	clr		second_byte
	clr		backup
	;ldi		second_byte,$DE 
;==============================================================
;			Sleep mode routine (its not used)
;==============================================================
;sleep_mode:
;	ldi 	temp,0x30		;turn on sleep mode (Power-down mode) 
;	out 	MCUCR,temp		;interrupt on low level.
;	ldi 	temp,0x40		;enable external interrupts (INT0)
;	out 	GIMSK,temp
;	sei						;enable global interrupts ready	
;	sleep					;go to sleep mode
;==============================================================

start:

	ldi		init_byte,first_byte;load init_byte with initial byte (recognition byte)
	rcall	receive				;receive data from PD1
	cp		rxbyte,init_byte	 ;compare received byte with recognition byte.
	brne	start				;if its different, go to start else ...

	rcall	receive
	mov		temp,rxbyte
	cp	 	temp,second_byte
	brne	check
	rjmp	next

check:
	inc		backup
	cp		backup,rxbyte
	breq	next
	mov		backup,rxbyte
	rjmp	start

;===================================================================================
next:
	mov		second_byte,rxbyte
	rcall	receive				;receive the second byte...
	mov		command,rxbyte 		;make backup this byte to command...
	rcall	receive     		;receive the 4th byte.
	eor		init_byte,command   ;make checksum of initial byte and command byte...
	eor		init_byte,second_byte
	cp		rxbyte,init_byte 	; compare checksum with 3rd received byte.
	brne	start				;if chechsum is different, go to receive new bytes.

	rcall	delay
	eor		output,command
	out		portb,output		;else, show the command to port B.	
	
	inc		second_byte			
	rjmp	start

;*************************************************************************
;					Receiving routine RxD
;*************************************************************************
receive:

	sbis	USR,RXC
	rjmp	receive
	in		Rxbyte,UDR

	ret

;***************************************************************************
;*			Delay routine
;***************************************************************************


delay:
	ldi		dcount2,255 
delay_:
	ldi		dcount,255
delay1:
	dec		dcount
	brne	delay1
	dec		dcount2
	brne	delay_
	ret


.db "(c) 09.06.2004 by Serasidis Vasilis. www.serasidis.gr"

